/* Copyright(C) 2004 Brazil

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _SEN_INV_H
#define _SEN_INV_H

#ifndef _SENNA_H
#include "senna_in.h"
#endif /* _SENNA_H */

#ifndef _SEN_SET_H
#include "set.h"
#endif /* _SEN_SET_H */

#ifndef _SEN_IO_H
#include "io.h"
#endif /* _SEN_IO_H */

#ifdef	__cplusplus
extern "C" {
#endif

typedef struct _sen_inv_entry sen_inv_entry;
typedef struct _sen_inv_updspec sen_inv_updspec;

sen_inv * sen_inv_create(const char *path, int flags, uint32_t initial_n_segments);
sen_inv * sen_inv_open(const char *path);
sen_rc sen_inv_close(sen_inv *inv);
sen_inv_entry * sen_inv_fetch(sen_inv *inv, uint32_t key);
sen_rc sen_inv_update(sen_inv *inv, uint32_t key, sen_inv_updspec *u, int hint);
sen_rc sen_inv_delete(sen_inv *inv, uint32_t key, sen_inv_updspec *u);

sen_inv_updspec * sen_inv_updspec_open(uint8_t type, uint32_t rid, int32_t score);
sen_rc sen_inv_updspec_close(sen_inv_updspec *u);
sen_rc sen_inv_updspec_add(sen_inv_updspec *u, int pos, int32_t weight,
			   int_least8_t boundary);

uint32_t sen_inv_estimate_size(sen_inv *inv, uint32_t key);

typedef struct {
  uint32_t size;
  int32_t mark;
  uint32_t *array;
} sen_inv_postings;

typedef struct {
  uint32_t rid;
  uint32_t brid;
  uint32_t srid;
  uint32_t tf;
  uint32_t score;
  uint32_t pos;
  uint32_t offset;
  int_least8_t boundary;
  uint8_t *sp;
  uint8_t *spe;
  int yet;
  sen_set *h;
  sen_io_win iw;
  struct _sen_inv_set_entry **hh;  
  struct _sen_inv_set_entry **hp;
  struct _sen_inv_set_entry **hpe;
} sen_inv_cursor;

sen_inv_cursor *sen_inv_cursor_open(sen_inv *inv, uint32_t key);
sen_rc sen_inv_cursor_next(sen_inv_cursor *c);
sen_rc sen_inv_cursor_get_property(sen_inv_cursor *c);
int sen_inv_cursor_get_postings(sen_inv_cursor *c, sen_inv_postings *p,
				int offset, int count);
sen_rc sen_inv_cursor_close(sen_inv_cursor *c);

void sen_inv_describe_term(sen_inv *inv, uint32_t key);

#ifdef __cplusplus
}
#endif

#endif /* _SEN_INV_H */
